import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import '../../mobile_ui.dart';
import '../../mobile_utils.dart';
import 'form_input_base.dart';

class DatePickerFormInput extends FormInputBase<DateTime> {
  final DateTime? firstDate;
  final DateTime? lastDate;
  final bool canClear;
  final CupertinoDatePickerMode? mode;

  static Future<void> _selectDate(
      BuildContext context,
      DateTime? initialValue,
      DateTime? firstDate,
      DateTime? lastDate,
      ValueChanged<DateTime?> onChanged,
      CupertinoDatePickerMode? mode) async {
    final dateTime = await showModalBottomSheet(
        context: context,
        builder: (_) {
          DateTime? dt = initialValue ?? DateTime.now();
          return Padding(
            padding: EdgeInsets.symmetric(vertical: 8),
            child: Column(
              children: [
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    TextButton(
                        onPressed: () => Navigator.of(context).pop(),
                        child: Text('Cancel')),
                    TextButton(
                        onPressed: () => Navigator.of(context).pop(dt),
                        child: Text('Save')),
                  ],
                ),
                Expanded(
                    child: CupertinoDatePicker(
                  mode: mode ?? CupertinoDatePickerMode.date,
                  maximumDate: lastDate,
                  minimumDate: firstDate,
                  initialDateTime: initialValue,
                  onDateTimeChanged: (value) {
                    dt = value;
                  },
                ))
              ],
            ),
          );
        });
    if (dateTime != null && dateTime != initialValue) onChanged(dateTime);
  }

  DatePickerFormInput(
      {DateTime? initialValue,
      ValueController<DateTime>? controller,
      this.canClear = false,
      this.firstDate,
      this.lastDate,
      this.mode = CupertinoDatePickerMode.dateAndTime,
      String? labelText,
      String? hintText,
      FormInputBackgroundStyle? backgroudStyle =
          defalutFormInputBackgroundStyle,
      bool? isDense,
      bool readOnly = false,
      bool enabled = true,
      bool isRequired = false,
      String format = formatDateTimeSG,
      FormFieldSetter<DateTime>? onSaved,
      ValueValidator<DateTime?>? validator,
      ValueChanged<DateTime?>? onChanged,
      AutovalidateMode? autovalidateMode,
      TextAlign labelAlign = TextAlign.left,
      FieldOrientation orientation = FieldOrientation.vertical,
      EdgeInsetsGeometry contentPadding = const EdgeInsets.all(8),
      EdgeInsetsGeometry margin = const EdgeInsets.symmetric(vertical: 8),
      ControlPosition controlPosition = ControlPosition.trailing,
      double spacingFlactor = 1,
      double? labelWidth,
      int errorMaxLines = 1})
      : assert(initialValue == null || controller == null),
        super(
            initialValue: initialValue,
            labelWidth: labelWidth,
            onSaved: onSaved,
            validator: validator,
            enabled: enabled,
            autovalidateMode: autovalidateMode,
            orientation: orientation,
            controlPosition: controlPosition,
            spacingFlactor: spacingFlactor,
            isDense: isDense,
            labelText: labelText,
            labelAlign: labelAlign,
            valueController: controller,
            isRequired: isRequired,
            contentPadding: contentPadding,
            margin: margin,
            errorMaxLines: errorMaxLines,
            onChanged: onChanged,
            builder: (field) {
              DatePickerFormInput widget = field.widget as DatePickerFormInput;
              final FormInputStateBase<DateTime?> state =
                  field as FormInputStateBase<DateTime?>;
              final theme = Theme.of(field.context);
              var text =
                  (field.value == null) ? '' : field.value!.formatBy(format);
              final icon = (canClear && field.value != null)
                  ? widget.buildIcon(theme, Icons.clear, onTap: () {
                      state.changeValue(null);
                    })
                  : widget.buildIcon(theme, Icons.calendar_today_outlined);
              Widget textWidget;
              if (hintText != null && hintText.isNotEmpty && (text.isEmpty)) {
                textWidget =
                    Text(hintText, style: theme.inputDecorationTheme.hintStyle);
              } else {
                textWidget = widget.buildText(theme, text);
              }
              final child = InkWell(
                  onTap: () {
                    _selectDate(field.context, field.value, firstDate, lastDate,
                        (dt) {
                      state.changeValue(dt);
                    }, mode);
                  },
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[textWidget, icon],
                  ));
              return widget.buildField(field, child,
                  backgroudStyle: backgroudStyle);
            });
}
