import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

const String formatDateSG = 'dd/MM/yyyy';
const String formatDateDayofWeekSG = 'dd/MM/yyyy EEEE';
const String formatDateTimeSG = 'dd/MM/yy HH:mm';
const String formatDateTimeAMSG = 'dd/MM/yy hh:mm aaa';
const String formatTimeSG = 'hh:mm aaa';

extension DateTimeExtension on DateTime {
  String formatBy([String pattern = formatDateSG]) {
    var formater = DateFormat(pattern);
    return formater.format(this);
  }

  DateTime date() {
    return DateTime(year, month, day);
  }

  DateTime addDuration(
      {int years = 0,
      int months = 0,
      int days = 0,
      int hours = 0,
      int minutes = 0,
      int seconds = 0,
      int milliseconds = 0,
      int microseconds = 0}) {
    var dt = this.add(Duration(
        days: days,
        hours: hours,
        minutes: minutes,
        seconds: seconds,
        milliseconds: milliseconds,
        microseconds: microseconds));
    int y = dt.year + years;
    int m = dt.month;
    if (months > 0) {
      final addMonths = months;
      y += addMonths ~/ 12;
      m += addMonths % 12;
      if (m > 12) {
        y += 1;
        m -= 12;
      }
    } else if (months < 0) {
      final addMonths = months.abs();
      y -= addMonths ~/ 12;
      m -= addMonths % 12;
      if (m < 0) {
        y -= 1;
        m += 12;
      }
    }
    return DateTime(y, m, dt.day, dt.hour, dt.minute, dt.second, dt.millisecond,
        dt.microsecond);
  }

  String formatTimeStamp(int timeStamp, Locale locale) {
    var dt = DateTime.fromMillisecondsSinceEpoch(timeStamp);
    var formatter = DateFormat.yMd(locale.toString()).add_Hm();
    return formatter.format(dt);
  }

  String formatTimeStampToShortDate(int timeStamp, Locale locale) {
    var dt = DateTime.fromMillisecondsSinceEpoch(timeStamp);
    var formatter = DateFormat.yMd(locale.toString());
    return formatter.format(dt);
  }

  String formatToShortDate(Locale locale) {
    var formatter = DateFormat.yMd(locale.toString());
    return formatter.format(this);
  }

  static DateTime fromTimeStamp(int timeStamp) {
    return DateTime.fromMillisecondsSinceEpoch(timeStamp);
  }

  static DateTime? fromJson(String json) {
    if (json.isEmpty) return null;
    var dt = DateTime.tryParse(json)!;
    return dt.toLocal();
  }

  String toJson() {
    return toUtc().toString();
  }
}
